#!/usr/sbin/perl
#
# EPcracking.pl v1.0.2
# Last modified August 4, 2003 16:13
# (c) Olaf R.P. Bininda-Emonds
#
# Usage 'perl EPcracking.pl inFile'
#
# Determines significantly moving events from a PAUP* logfile containing apomorphy list (inFile)

$inFile = MacPerl::Ask("Enter name of PAUP* logfile","EventPars.log");
$AllChange = MacPerl::Answer("Changes to be included?", "All", "Unequivocal");
$TrackChange = MacPerl::Answer("Track change categories?", "Yes", "No");
$verbosePick = MacPerl::Answer("Verbose output?", "No", "Yes");
	$verbose = 1 if ($verbosePick == 0);

($outFile) = split(/\./,$inFile,0);
$outApo = $outFile.".apo";
$changeFile = $outFile.".chg";

#Subroutine to distill information from PAUP* logfile

	print "Distilling information from PAUP* logfile\n\n" if ($verbose);
	open (fIN,"<$inFile");
	open (fOUT,">$outApo");
	open (fCHANGE,">$changeFile") if ($TrackChange);
	
	$OptCount = 0;
	if (not $AllChange)
		{
		print "Optimization criterion: unequivocal\n" if ($verbose);
		print fOUT "Optimization criterion: unequivocal\n\n";
		print fCHANGE "Optimization criterion: unequivocal\n\n" if ($TrackChange);			
		}
	
	while (<fIN>)
		{
		chop;
		$_ =~ s/^\s+//g;	#Strip (any) leading white space
		if (m/total characters:/ and not $NumChar)	# Derive total number of events
			{
			@CharLine = split;
			$NumChar = $CharLine[1];
			$NumEvent = int(sqrt($NumChar*2));
			until (($NumEvent*$NumEvent - $NumEvent) / 2 == $NumChar)
				{
				$NumEvent++;
				}
			}
		if (m/are excluded/ and not $NumChar)	# Derive total number of events
			{
			@CharLine = split;
			$ExclChar = $CharLine[0];
			}
		if (m/included characters:/ and not $NumChar)	# Derive total number of events
			{
			@CharLine = split;
			$InclChar = $CharLine[3];
			$NumChar = $ExclChar + $InclChar;
			$NumEvent = int(sqrt($NumChar*2));
			until (($NumEvent*$NumEvent - $NumEvent) / 2 == $NumChar)
				{
				$NumEvent++;
				}
			}
		if (m/optimization/ and $AllChange)
			{
			@OptLine = split(/\(/);
			$OptLine[1] =~ s/\)//;
			$OptCrit = $OptLine[1];
			print "\nOptimization criterion: $OptCrit\n" if ($verbose);
			print fOUT "\nOptimization criterion: $OptCrit\n\n";
			print fCHANGE "\nOptimization criterion: $OptCrit\n\n" if ($TrackChange);
			}
		if (m/Tree description:/)
			{
			$OptCount++;
			}
		unless ($OptCount > 1 and not $AllChange)	# Look at first apomorphy list only if using unequivocal changes
			{
			if (m/Apomorphy lists:/)
				{
				print "\n$_\n\n" if ($verbose);
				}
			elsif (m/node_/ or eof)
				{
				if (($node or eof) and $TrackChange)	# Subroutine to print tracked changes
					{
					print "\n\tTracked changes:\t" if ($verbose);
					print fCHANGE "$node\t";
					for ($from = 0; $from <= 2; $from++)
						{
						for ($to = 0; $to <= 2; $to++)
							{
							unless ($from == $to)
								{
								if (not $ChangeTotal[$from][$to])
									{
									$ChangeTotal[$from][$to] = 0;
									}
								print "$from$to: $ChangeTotal[$from][$to]\t" if ($verbose);
								print fCHANGE "$ChangeTotal[$from][$to]\t";
								$ChangeTotal[$from][$to] = 0;
								}
							}
						}
					print "\n\n" if ($verbose);
					print fCHANGE "\n";
					}
				unless (eof)
					{
					($node,$EP,$wt,$CI,$apo) = split(/\s\s+/);
					if ($node =~ /\s\d+/)	# Safety when node name and next column are separated by only a single space
						{
						@NewNode = split/\s/,$node;
						$EP = pop(@NewNode);
						$node = join(" ", @NewNode);
						$fromApo = substr($CI,0,1);
						$toApo = substr($CI,-1,1);
						$Change = substr($CI,3,1);
						}
					else
						{
						$fromApo = substr($apo,0,1);
						$toApo = substr($apo,-1,1);
						$Change = substr($apo,3,1);
						}
					print "$node\n" if ($verbose);
					print fOUT "$node\n";
					if ($Change eq "-")
						{
						unless (not $AllChange)
							{
							print "\t$EP\t$fromApo\t$toApo\n" if ($verbose);
							print fOUT "\t$EP\t$fromApo\t$toApo\n";
							$ChangeTotal[$fromApo][$toApo]++;
							}
						}
					else
						{
						print "\t$EP\t$fromApo\t$toApo\n" if ($verbose);
						print fOUT "\t$EP\t$fromApo\t$toApo\n";
						$ChangeTotal[$fromApo][$toApo]++;
						}
					}
				}
			elsif (m/>/)
				{
				($EP,$wt,$CI,$apo) = split(/\s\s+/);
				$fromApo = substr($apo,0,1);
				$toApo = substr($apo,-1,1);
				$Change = substr($apo,3,1);
				if ($Change eq "-")
					{
					unless (not $AllChange)
						{
						print "\t$EP\t$fromApo\t$toApo\n" if ($verbose);
						print fOUT "\t$EP\t$fromApo\t$toApo\n";
						$ChangeTotal[$fromApo][$toApo]++;
						}
					}
				else
					{
					print "\t$EP\t$fromApo\t$toApo\n" if ($verbose);
					print fOUT "\t$EP\t$fromApo\t$toApo\n";
					$ChangeTotal[$fromApo][$toApo]++;
					}
				}
			}
		}
	close fIN;
	close fOUT;
		MacPerl::SetFileInfo("R*ch", "TEXT", $outApo);
	if ($TrackChange)
		{
		close fCHANGE;
		MacPerl::SetFileInfo("R*ch", "TEXT", $changeFile);
		}

# End of subroutine

# Subroutine to crack synapomorphies

	print "Cracking synapomorphies ...\n\n" if ($verbose);
	($outCrack) = split(/\./,$outApo,0);
	$outCrack .= ".crk";
	
	open (APOLIST,"<$outApo");
	open (RESULTS,">$outCrack");
	
	$CharNum = 1;
	for ($Row = 2; $Row <= $NumEvent; $Row++)	# Assign constituent event to each event-pair
		{
		for ($Column = 1; $Column < $Row; $Column++)
			{
			$ColumnEvent{$CharNum} = $Column;
			$RowEvent{$CharNum} = $Row;
			$CharNum++
			}
		}
	
	$NewNode = $OldNode = "";
	$SumTRM = $MeanTRM = $MedianTRM = 0;
	for (1..$NumEvent)
		{
		$TRMRow[$_] = $TRMColumn[$_] = $TRMabs[$_] = $AbsMove[$_] = 0;
		}
	@TRM = @SortedTRM = @AdjTRM = @AdjAbsMove = @Status = @Selected = @J = ();
	
	while (<APOLIST>)
		{
		chop;
		if (m/Optimization criterion:/)
			{
			print RESULTS "$_\n\n";
			}
		else
			{
			if (m/>/ or eof)
				{
				$NewNode = $_;
				if ($OldNode)	# Process and output information for previous node
					{
					print "\nRelative and absolute movements (first round of selection)\n\n" if ($verbose);
					for (1..$NumEvent)		# Calculate relative and absolute movements for each event
						{
#						if ($_ == $NumEvent)	# BUG: Would avoid unitialized value
#							{
#							$TRM[$_] = $TRMRow[$_];
#							}
#						else
#							{
#							$TRM[$_] = $TRMRow[$_] - $TRMColumn[$_];
#							}
						$TRM[$_] = $TRMRow[$_] - $TRMColumn[$_];
						$SumTRM += abs($TRM[$_]);
						$TRMabs[$_] = abs($TRM[$_]);
						print "Event: $_\tRow: $TRMRow[$_]\tColumn: $TRMColumn[$_]\tTRM: $TRM[$_]\tAbs: $AbsMove[$_]\n" if ($verbose);
						}
					print "\n" if ($verbose);
					$MeanTRM = $SumTRM / $NumEvent;
					@SortedTRM = sort {$a <=> $b} @TRMabs;	# BUG: Note that $TRMabs[0] is unitialized
					if (int($NumEvent/2)*2 == $NumEvent)	# Even number of events
						{
						$MedianTRM = ($SortedTRM[$NumEvent/2] + $SortedTRM[($NumEvent/2)+1]) / 2;
						}
					else
						{
						$MedianTRM = $SortedTRM[($NumEvent/2)+0.5];
						}
					print "Cutoff values\n\nMean: $MeanTRM\tMedian: $MedianTRM\n" if ($verbose);
					for (1..$NumEvent)
						{
						if ($TRMabs[$_] > $MedianTRM)
							{
							$Status[$_] = "selected";
							push @Selected, $_;
							}
						else
							{
							$Status[$_] = "1st round failure";
							}
						}
					
					print "First round selected events: @Selected\n\n" if ($verbose);
					
					# Subroutine for second round of selection
					
						print "Second round of selection\n\n" if ($verbose);
						foreach $Event (@Selected)
							{
							print "Event: $Event\n" if ($verbose);
							foreach $OtherEvent (@Selected)
								{
								unless ($Event == $OtherEvent)
									{
									if ($MoveWith[$Event][$OtherEvent])		# Event moves as a column event
										{
										$SecondTRM[$Event] += $MoveWith[$Event][$OtherEvent]*-1;
										$SecondAbsMove[$Event] += abs($MoveWith[$Event][$OtherEvent]);
										print "\tColumn movement of $MoveWith[$Event][$OtherEvent] with $OtherEvent\n" if ($verbose);
										$MoveWith[$OtherEvent][$Event] = 0;		# Clear reciprocal movement
										}
									elsif ($MoveWith[$OtherEvent][$Event])	# Event moves as a row event
										{
										$SecondTRM[$Event] += $MoveWith[$OtherEvent][$Event];
										$SecondAbsMove[$Event] += abs($MoveWith[$OtherEvent][$Event]);
										print "\tRow movement of $MoveWith[$OtherEvent][$Event] with $OtherEvent\n" if ($verbose);
										$MoveWith[$Event][$OtherEvent] = 0;		# Clear reciprocal movement
										}
									}
								}
							$AdjTRM[$Event] = $TRM[$Event] - $SecondTRM[$Event];
							$AdjAbsMove[$Event] = $AbsMove[$Event] - $SecondAbsMove[$Event];
							if ($AdjTRM[$Event] != 0)
								{
								if ($AdjTRM[$Event] < -1)
									{
									$Status[$Event] = "early";
									}
								elsif ($AdjTRM[$Event] > 1)
									{
									$Status[$Event] = "late";
									}
								else
									{
									$Status[$Event] = "twin";
									}
								for (1..$NumEvent)
									{
									$TargetEvent = $_;
									if ($MoveWith[$Event][$TargetEvent] and $MoveWith[$TargetEvent][$Event])
										{
										$MoveList[$Event] .= $TargetEvent."\t";
										unless ($AdjAbsMove[$Event] == 0)
											{
											$J[$Event] = $AdjTRM[$Event] / $AdjAbsMove[$Event];
											}
										}
									}				
								$MoveList[$Event] =~ s/\s+$//;
								}
							else
								{
								$Status[$Event] = "hitchhiker";
								$J[$Event] = "undefined";
								}
							print "\tAdjTRM: $AdjTRM[$Event]\tAdj Abs: $AdjAbsMove[$Event]\tJ: $J[$Event]\n\n" if ($verbose);
							}
					
					# End of subroutine
		
					print "Summary of movement for: $OldNode\n\n" if ($verbose);
					print RESULTS "Node name: $OldNode\n";
					print RESULTS "Node\tEvent\tStatus\tJ\tRelative to:\n";
					for (1..$NumEvent)
						{
						print "Event: $_\tStatus: $Status[$_]" if ($verbose);
						print RESULTS "$OldNode\t$_\t$Status[$_]";
						if ($Status[$_] eq "early")
							{
							print "\tJ: $J[$_]\tRelative to: $MoveList[$_]\n" if ($verbose);
							print RESULTS "\t$J[$_]\t$MoveList[$_]\n";
							push @early, $_;
							}
						elsif ($Status[$_] eq "late")
							{
							print "\tJ: $J[$_]\tRelative to: $MoveList[$_]\n" if ($verbose);
							print RESULTS "\t$J[$_]\t$MoveList[$_]\n";
							push @late, $_;
							}
						elsif ($Status[$_] eq "twin")
							{
							print "\t\tRelative to: $MoveList[$_]\n" if ($verbose);
							print RESULTS "\t\t$MoveList[$_]\n";
							}
						else
							{
							print "\n" if ($verbose);
							print RESULTS "\n";
							}
						}
					$numEarly = @early;
					print "\n\tearly ($numEarly): " if ($verbose);
					print RESULTS "\n\tearly ($numEarly): ";
					if (@early)
						{
						print "@early\n" if ($verbose);
						print RESULTS "@early\n";
						}
					else
						{
						print "none\n" if ($verbose);
						print RESULTS "none\n";
						}
					$numLate = @late;
					print "\tlate ($numLate): " if ($verbose);
					print RESULTS "\tlate ($numLate): ";
					if (@late)
						{
						print "@late\n\n" if ($verbose);
						print RESULTS "@late\n\n";
						}
					else
						{
						print "none\n\n" if ($verbose);
						print RESULTS "none\n\n";
						}
					@TRM = @AdjTRM = @AbsMove = @AdjAbsMove = @Status = @Selected = @MoveWith = @J = ();
					@SecondTRM = @SecondAbsMove = @MoveList = ();
					@early = @late = ();
					$SumTRM = $MedianTRM = 0;
					for (1..$NumEvent)
						{
						$TRMRow[$_] = $TRMColumn[$_] = $TRMabs[$_] = $AbsMove[$_] = 0;
						}
					}
				if (not eof)
					{
					print "Apomorphy list for: $NewNode\n\n" if ($verbose);
					}
				$OldNode = $NewNode;
				}
			else
				{
				($Char,$FromState,$ToState) = split;	# BUG: Will choke on blank lines
				if ($FromState - $ToState > 0)
					{
					$RelMove = -1;
					}
				else
					{
					$RelMove = 1;
					}
				$TRMColumn[$ColumnEvent{$Char}] += $RelMove;
				$TRMRow[$RowEvent{$Char}] += $RelMove;
				$AbsMove[$ColumnEvent{$Char}]++;
				$AbsMove[$RowEvent{$Char}]++;
				$MoveWith[$ColumnEvent{$Char}][$RowEvent{$Char}] = $RelMove;
				$MoveWith[$RowEvent{$Char}][$ColumnEvent{$Char}] = $RelMove;
				print "Character: $Char\tRow: $RowEvent{$Char}\tColumn: $ColumnEvent{$Char}\tFrom: $FromState\tTo: $ToState\tRel move: $RelMove\n" if ($verbose);
				}
			}
		}
	
	close APOLIST;
	close RESULTS;
		MacPerl::SetFileInfo("R*ch", "TEXT", $outCrack);

# End of subroutine

# Subroutine to determine moving events common to all character reconstructions

	print "\nDetermining common movements ...\n\n" if ($verbose);
	($outConCrack) = split(/\./,$outCrack,0);
	$outConCrack1 = $outConCrack.".evt";
	$outConCrack2 = $outConCrack.".nod";
	$outConCrack3 = $outConCrack.".opt";
	
	@EarlyMoveCount = @LateMoveCount = ();
	@EarlyNode = @EarlyRelTo = @EarlyOpt = @LateNode = @LateRelTo = @LateOpt = ();

	open (CRACK,"<$outCrack");
	
	while (<CRACK>)
		{
		chop;
		if (m/Optimization criterion:/)
			{
			@OptList = split(/:/);
			$OptList[1] =~ s/\s+//g;
			$OptCrit = $OptList[1];
			}
		if (m/Node name:/)
			{
			($header,$NodeName) = split(/:/);
			$NodeName =~ s/^\s+//;
			if ($TimesPresent{$NodeName} == 0)
				{
				$NodeCount++;
				$NodeID{$NodeName} = $NodeCount;
				push @NameList, $NodeName;
				}
			$TimesPresent{$NodeName}++;
			print "Node: $NodeName\tPresent: $TimesPresent{$NodeName}\n" if ($verbose);
			}
		else
			{
			unless (m/early:/ or m/late:/)
				{
				($NodeRead,$EventRead,$StatusRead,$JRead,@MoveListRead) = split(/\t/);
				if ($EventRead =~ /\d/)		# Only when reading a data line, not a header line
					{
					print "\tEvent: $EventRead\n" if ($verbose);
					if ($EventRead > $NumEvent)
						{
						$NumEvent = $EventRead;
						}
					unless ($NoChange[$NodeID{$NodeRead}][$EventRead] == 1)		# Only check events that have moved
						{
						if (abs($JRead) != 1)	# Mark events that have not moved
							{
							print "\t\tEvent has not moved, J <> +/- 1\n" if ($verbose);
							$NoChange[$NodeID{$NodeRead}][$EventRead] = 1;
							}
						else
							{
							if ($StatusRead eq "early")
								{
								push @{$EarlyNode[$EventRead]}, $NodeRead;
								push @{$EarlyRelTo[$EventRead]}, join(" ", @MoveListRead);
								push @{$EarlyOpt[$EventRead]}, $OptCrit;
								$EarlyMoveCount[$EventRead]++
								}
							if ($StatusRead eq "late")
								{
								push @{$LateNode[$EventRead]}, $NodeRead;
								push @{$LateRelTo[$EventRead]}, join(" ", @MoveListRead);
								push @{$LateOpt[$EventRead]}, $OptCrit;
								$LateMoveCount[$EventRead]++
								}
							if ($TimesPresent{$NodeRead} == 1)	# Set working values for first read
								{
								print "\t\tAssigning working values ...\n" if ($verbose);
								$Status[$NodeID{$NodeRead}][$EventRead] = $StatusRead;
								$J[$NodeID{$NodeRead}][$EventRead] = $JRead;
								$MoveList[$NodeID{$NodeRead}][$EventRead] = join(" ", @MoveListRead);
								print "\t\t\tStatus: $Status[$NodeID{$NodeRead}][$EventRead]\n" if ($verbose);
								print "\t\t\tJ: $J[$NodeID{$NodeRead}][$EventRead]\n" if ($verbose);
								print "\t\t\tMoveList: $MoveList[$NodeID{$NodeRead}][$EventRead]\n" if ($verbose);
								}
							else	# Check read values versus working values
								{
								if ($StatusRead ne $Status[$NodeID{$NodeRead}][$EventRead] or $JRead != $J[$NodeID{$NodeRead}][$EventRead])		# Clear working values in case of discrepancy
									{
									$NoChange[$NodeID{$NodeRead}][$EventRead] = 1;
									$Status[$NodeID{$NodeRead}][$EventRead] = "";
									$J[$NodeID{$NodeRead}][$EventRead] = "";
									$MoveList[$NodeID{$NodeRead}][$EventRead] = "";
									print "\t\tStatus or J values do not match\n" if ($verbose);
									}
								else	# Get intersection of MoveLists
									{
									print "\t\tComputing intersection of MoveLists ...\n" if ($verbose);
									@WorkingMoveList = split(/ /,$MoveList[$NodeID{$NodeRead}][$EventRead]);
									@union = @intersection = @difference = ();	# Calculate intersection of MoveLists
									%count = ();
									foreach $element (@MoveListRead, @WorkingMoveList)
										{
										$count{$element}++
										}
									foreach $element (keys %count)
										{
										push @union, $element;
										push @{ $count{$element} > 1 ? \@intersection : \@difference }, $element;
										}
									@intersection = sort {$a <=> $b} @intersection;
									@difference = sort {$a <=> $b} @difference;
									print "\t\t\tIntersection: @intersection\n" if ($verbose);
									print "\t\t\tDifference: @difference\n" if ($verbose);
									if ($#intersection < 0)		# No common movements
										{
										$NoChange[$NodeID{$NodeRead}][$EventRead] = 1
										}
									else
										{
										$MoveList[$NodeID{$NodeRead}][$EventRead] = join(" ", @intersection);
										}
									}
								}
							}
						}
					}
				}
			}
		}
	close CRACK;

	print "\nMovement for each event by node\n\n" if ($verbose);
	open (RESULTS,">$outConCrack1");
		print "Event\t" if ($verbose);
		print RESULTS "Event\t";
		foreach $node (@NameList)
			{
			print "$node\t" if ($verbose);
			print RESULTS "$node\t";
			}
		print "\n" if ($verbose);
		print RESULTS "\n";
		for ($Event = 1; $Event <= $NumEvent; $Event++)
			{
			print "$Event\t" if ($verbose);
			print RESULTS "$Event\t";
			foreach $node (@NameList)
				{
				if ($NoChange[$NodeID{$node}][$Event])
					{
					print "\t" if ($verbose);
					print RESULTS "\t";
					}
				else
					{
					if ($Status[$NodeID{$node}][$Event] eq "early")
						{
						print "early\t" if ($verbose);
						print RESULTS "early\t";
						}
					if ($Status[$NodeID{$node}][$Event] eq "late")
						{
						print "late\t" if ($verbose);
						print RESULTS "late\t";
						}
					}
				}
			print "\n" if ($verbose);
			print RESULTS "\n";
			}
	close RESULTS;
		MacPerl::SetFileInfo("R*ch", "TEXT", $outConCrack1);

	print "\nMovement at each node\n\n" if ($verbose);
	open (RESULTS,">$outConCrack2");
		foreach $node (@NameList)
			{
			@early = @late = ();
			print "Node: $node\n" if ($verbose);
			print RESULTS "\nNode: $node\n";
			print RESULTS "Event\tStatus\tJ\tMoveList\n";
			for (1..$NumEvent)
				{
				unless ($NoChange[$NodeID{$node}][$_])
					{
					print "\tEvent: $_\tStatus: $Status[$NodeID{$node}][$_]\tJ: $J[$NodeID{$node}][$_]\tMoveList: $MoveList[$NodeID{$node}][$_]\n" if ($verbose);
					print RESULTS"$_\t$Status[$NodeID{$node}][$_]\t$J[$NodeID{$node}][$_]\t$MoveList[$NodeID{$node}][$_]\n";
					if ($Status[$NodeID{$node}][$_] eq "early")
						{
						push @early, $_;
						}
					if ($Status[$NodeID{$node}][$_] eq "late")
						{
						push @late, $_;
						}
					}
				}
			$numEarly = @early;
			print "\n\tearly ($numEarly): " if ($verbose);
			print RESULTS "\n\tearly ($numEarly): ";
			if (@early)
				{
				print "@early\n" if ($verbose);
				print RESULTS "@early\n";
				}
			else
				{
				print "none\n" if ($verbose);
				print RESULTS "none\n";
				}
			$numLate = @late;
			print "\tlate ($numLate): " if ($verbose);
			print RESULTS "\tlate ($numLate): ";
			if (@late)
				{
				print "@late\n\n" if ($verbose);
				print RESULTS "@late\n\n";
				}
			else
				{
				print "none\n\n" if ($verbose);
				print RESULTS "none\n\n";
				}
			}
	close RESULTS;
		MacPerl::SetFileInfo("R*ch", "TEXT", $outConCrack2);
	
	print "\nMovement for each event according to optimization criterion\n\n" if ($verbose);
	open (RESULTS,">$outConCrack3");
		for ($Event = 1; $Event <= $NumEvent; $Event++)
			{
			print "Event: $Event\n" if ($verbose);
			print RESULTS "Event: $Event\n";
			print "\tEarly movement:\n" if ($verbose);
			print RESULTS "\tEarly movement:\n";
			if (not $EarlyMoveCount[$Event])
				{
				print "\t\tnone\n" if ($verbose);
				print RESULTS "\t\tnone\n";
				}
			else
				{
				for ($count = 0; $count < $EarlyMoveCount[$Event]; $count++)
					{
					print "\t\t$EarlyNode[$Event][$count]\t$EarlyRelTo[$Event][$count]\t$EarlyOpt[$Event][$count]\n" if ($verbose);
					print RESULTS "\t\t$EarlyNode[$Event][$count]\t$EarlyRelTo[$Event][$count]\t$EarlyOpt[$Event][$count]\n";
					}
				}
			print "\n\tLate movement:\n" if ($verbose);
			print RESULTS "\tLate movement:\n";
			if (not $LateMoveCount[$Event])
				{
				print "\t\tnone\n\n" if ($verbose);
				print RESULTS "\t\tnone\n\n";
				}
			else
				{
				for ($count = 0; $count < $LateMoveCount[$Event]; $count++)
					{
					print "\t\t$LateNode[$Event][$count]\t$LateRelTo[$Event][$count]\t$LateOpt[$Event][$count]\n" if ($verbose);
					print RESULTS "\t\t$LateNode[$Event][$count]\t$LateRelTo[$Event][$count]\t$LateOpt[$Event][$count]\n";
					}
				print "\n" if ($verbose);
				print RESULTS "\n";
				}
			}	
	close RESULTS;
		MacPerl::SetFileInfo("R*ch", "TEXT", $outConCrack3);
	
# End of subroutine
